#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <dirent.h>
#include <rsbac/types.h>
#include <rsbac/getname.h>
#include <rsbac/syscalls.h>
#include <rsbac/error.h>
#include <rsbac/helpers.h>
#include <rsbac/aci_data_structures.h>

const char   set_prog[] = "auth_set_cap";
int recurse = 0;
int verbose = 0;
/* default max number of cap entries per file */
int maxnum = 20;

int process(char * name, FILE * tfile)
  {
    int res = 0;
    char tmp1[120];
    enum rsbac_target_t type;
    char type_n[5] = "";
    int i;
    struct stat buf;
    rsbac_uid_t caplist[maxnum];

    if (lstat(name,&buf))
      {
        fprintf(stderr, "stat for file %s returned error: %s\n",
               name,
               strerror(errno));
        return(1);
      }
    if(S_ISDIR(buf.st_mode))
      {
        type = T_DIR;
      }
    else
      if(S_ISREG(buf.st_mode))
        {
          type = T_FILE;
          strcpy(type_n,"FILE");
        }
      else
        {
          if(verbose)
            printf("%s is neither file nor dir, skipped\n",
                   name);
          return(-1);
        }
    if(verbose)
      printf("Processing %s '%s'\n", type_n, name);
    if(type == T_FILE)
      {
        res = rsbac_auth_get_f_caplist(name, caplist, maxnum);
        if(res<0)
          {
            if((res > -RSBAC_EPERM) || (res < -RSBAC_EDECISIONMISMATCH))
              fprintf(stderr, "%s: %s\n",
                      name,
                      strerror(-res));
            else
              {
                get_error_name(tmp1,res);
                fprintf(stderr, "%s: %s\n",
                        name,
                        tmp1);
              }
          }
        else
          {
            if(verbose)
              printf("%s: %i caps\n", name, res);
            for(i=0;i<res;i++)
              {
                fprintf(tfile,
                        "%s FILE add %s %i\n",
                        set_prog,
                        name,
                        caplist[i]);
              }
          }
      }
    if (   (type == T_DIR)
        && recurse)
      {
        DIR * dir_stream_p;
        struct dirent * dirent_p;
        char name2[512];

        if(S_ISLNK(buf.st_mode))
          return(0);
        if(!(dir_stream_p = opendir(name)))
          {
            fprintf(stderr, "opendir for dir %s returned error: %s\n",
                   name,
                   strerror(errno));
            return(-2);
          }
        while((dirent_p = readdir(dir_stream_p)))
          {
            if(   (strcmp(".",dirent_p->d_name))
               && (strcmp("..",dirent_p->d_name)) )
              {
                strcpy(name2,name);
                strcat(name2,"/");
                strcat(name2,dirent_p->d_name);
                process(name2, tfile);
              }
          }
        closedir(dir_stream_p);
      }
    return(0);
  }

int main(int argc, char ** argv)
{
  int res = 0;
  int i;
  FILE * tfile;
  char * progname;

  progname = argv[0];
  if ((argc > 1) && (!strcmp("-r",argv[1])) )
    {
      recurse=1;
      argv++;
      argc--;
    }  
  if ((argc > 1) && (!strcmp("-v",argv[1])) )
    {
      verbose=1;
      argv++;
      argc--;
    }  
  if (argc > 2)
    {
      if (!(tfile=fopen(argv[1],"w")))
        {
          fprintf(stderr, "opening target file returned error: %s\n",
                  strerror(errno));
        }
      printf("%s: %i targets", progname, argc - 2);
      if(recurse)
        printf(" - recursing");
      printf("\n");
      for (i=1;i < (argc-1);i++)
        {
          process(argv[i+1],tfile);
        }
      fclose(tfile);
    }
  else
    {
      printf("%s (RSBAC %s)\n***\n", argv[0], VERSION);
      printf("Use: %s [-r] [-v] target-file file/dirname(s)\n", progname);  
      printf("- should be called by root with all rsbac modules switched off,\n");
      printf("- -r = recurse in subdirs, -v = verbose, no symlinks followed,\n");
      printf("- target-file = name of restore script to be created,\n");
    }
  return (res);
}
